import os
import uuid


class IdTool(object):
    def __init__(self):
        self.uuid_node = os.getpid() ^ uuid.getnode()
        self.clock_seq = 0

    @classmethod
    def get_timestamp(cls, struuid: str):
        return (uuid.UUID(struuid).time - 0x01b21dd213814000) / 1e7

    def gen_uuid(self):
        self.clock_seq += 1
        return str(uuid.uuid1(self.uuid_node, self.clock_seq))


idtool = IdTool()



from enum import Enum
from pydantic import BaseModel, Field


class HeartbeatEventType(str, Enum):
    NEW = 'NEW'
    LOST = 'LOST'


class HeartbeatEvent(BaseModel):
    event_type: HeartbeatEventType = Field(HeartbeatEventType.NEW)
    namespace: str = Field("namespace")
    service_name: str = Field("service_name")
    host_ip: str = Field("0.0.0.0")
    host_name: str = Field("host_name")
    timestamp: int = Field(0)



from time import sleep

from rosetta_dispatcher.dispatch_server import DispatchServer
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel
from rosetta_dispatcher.model.dispatch_types import DispatchResponseStatus

host = '61.160.36.168'
port = 4032

ds = DispatchServer(redis_host=host, redis_port=port)

count = 0
while True:
    result = ds.fetch('NMT_RPC_QUEUE_DEV', batch_count=16)
    print(f'fetched: {len(result)}')
    response_list = [DispatchResponseModel(correlation_id=request.correlation_id, status=DispatchResponseStatus.OK,
                                         data=request.data) for request in result]

    ds.batch_send_response(result, response_list)





from pydantic import BaseModel, Field

from rosetta_dispatcher.model.dispatch_types import DispatchRequestType


class DispatchRequestModel(BaseModel):
    content_type: DispatchRequestType = Field(DispatchRequestType.JSON)
    reply_to: str = Field(None)
    correlation_id: str = Field(None)
    timeout: int = Field(None)
    data: str = Field(None)


import json

from rosetta_dispatcher.idtool import idtool
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel
from rosetta_dispatcher.model.dispatch_types import DispatchRequestType, DispatchResponseStatus
import aioredis


class AioDispatchClient(object):
    @classmethod
    async def create(cls, redis_host: str, redis_port: int):
        result = AioDispatchClient(redis_host, redis_port)
        result.pool = await aioredis.create_pool(
            f'redis://{redis_host}:{redis_port}', encoding='utf-8',
            minsize=1, maxsize=16)
        # result.redis = await aioredis.create_redis(f'redis://{redis_host}:{redis_port}', encoding='utf-8')
        return result

    def __init__(self, redis_host: str, redis_port: int):
        self.redis_host = redis_host
        self.redis_port = redis_port
        self.pool = None

    async def process(self, service_queue: str, data: str, timeout: int = 500):
        reply_to = idtool.gen_uuid()
        correlation_id = idtool.gen_uuid()
        request = {
            'content_type': DispatchRequestType.JSON,
            'reply_to': reply_to,
            'correlation_id': correlation_id,
            'timeout': timeout,
            'data': data
        }

        with await self.pool as conn:
            # with await self.pool as conn:  # low-level redis connection
            await conn.execute(b'RPUSH', service_queue, json.dumps(request, ensure_ascii=False))
            result = await conn.execute(b'BLPOP', reply_to, timeout)
            if result and result[0] == reply_to:
                response = DispatchResponseModel.parse_obj(json.loads(result[1]))
                if response.correlation_id == correlation_id and response.status == DispatchResponseStatus.OK:
                    return response.data
        return None



import time

from heartbeat.heartbeat_observer import HeartbeatObserver
from heartbeat.heatbeat_reporter import HeartbeatReporter
from heartbeat.model.heartbeat_event import HeartbeatEvent

host = '61.160.36.168'
port = 4032


def callback(result: HeartbeatEvent):
    print("test", result)




while True:
    time.sleep(1)



# encoding: UTF-8
import json
import threading
import time
from datetime import datetime, timedelta
import redis

import schedule
import time


class HeartbeatReporter:
    def __init__(self,
                 namespace: str,
                 service_name: str,
                 host_ip: str,
                 host_name: str,
                 redis_host: str,
                 redis_port: str,
                 interval: int = 60):
        self.host_ip = host_ip
        self.host_name = host_name
        self.redis_host = redis_host
        self.redis_port = redis_port
        self.running = False
        self.interval = interval
        self.namespace = namespace
        self.service_name = service_name
        self.pool = redis.ConnectionPool(host=redis_host, port=redis_port, decode_responses=True)

        schedule.every(self.interval).seconds.do(self.send_report)

        self.thread = threading.Thread(target=self.job_thread_fun)
        self.thread.start()

    def job_thread_fun(self):
        while True:
            schedule.run_pending()
            time.sleep(1)

    def send_report(self):
        if self.running:
            r = redis.Redis(connection_pool=self.pool)

            data = {
                "service_name": self.service_name,
                "host_ip": self.host_ip,
                "host_name": self.host_name
            }

            mappings = {json.dumps(data, ensure_ascii=False): int(time.time())}
            r.zadd(self.namespace, mappings)

    def start(self):
        self.running = True

    def stop(self):
        self.running = False



import os
from time import sleep

from rosetta_dispatcher.dispatch_server import DispatchServer
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel
from rosetta_dispatcher.model.dispatch_types import DispatchResponseStatus

def dispatch_server():
    host = '61.160.36.168'
    port = 4032

    ds = DispatchServer(redis_host=host, redis_port=port)

    count = 0
    while True:
        print('hello world!!!')
        result = ds.fetch('NMT_RPC_QUEUE_DEV', batch_count=16)
        print(f'fetched: {len(result)}')
        response_list = [DispatchResponseModel(correlation_id=request.correlation_id, status=DispatchResponseStatus.OK,
                                             data=request.data) for request in result]

        ds.batch_send_response(result, response_list)

def http_server():
    import uvicorn
    from fastapi import FastAPI

    app = FastAPI()

    @app.get("/health")
    async def health():
        return {"code": 0}

    uvicorn.run(app, host="0.0.0.0", port=8000)



def main():
    # Creating child process using fork
    processid = os.fork()
    if processid:
        # This is the parent process
        # Closes file descriptor w
        print("Parent running")
        dispatch_server()
    else:
        # This is the child process
        print("Child running")
        http_server()

main()










# encoding: UTF-8
import json
import threading
import time
from datetime import datetime, timedelta
from typing import List

import redis

import schedule
import time

from heartbeat.model.heartbeat_event import HeartbeatEvent, HeartbeatEventType


class HeartbeatObserver:
    def __init__(self,
                 namespaces: List[str],
                 redis_host: str,
                 redis_port: str,
                 interval: int = 60,
                 callback: callable = None):

        self.redis_host = redis_host
        self.redis_port = redis_port
        self.running = True
        self.interval = interval
        self.namespaces = namespaces
        self.pool = redis.ConnectionPool(host=redis_host, port=redis_port, decode_responses=True)

        schedule.every(self.interval).seconds.do(self.observe_report)
        self.thread = threading.Thread(target=self.job_thread_fun)
        self.thread.start()

        self.service_mapping = {namespace: {} for namespace in self.namespaces}
        self.callback = callback

    def job_thread_fun(self):
        while True:
            schedule.run_pending()
            time.sleep(1)

    def observe_report(self):
        if self.running:
            r = redis.Redis(connection_pool=self.pool)
            now = int(time.time())
            for namespace in self.namespaces:
                if self.callback:
                    items = r.zrangebyscore(namespace, min=now - self.interval, max=now + 1, withscores=True)
                    newdict = {item[0]: item[1] for item in items}
                    newset = {item[0] for item in items}

                    olddict = self.service_mapping[namespace]
                    oldset = {key for key in olddict}

                    for stritem in newset - oldset:
                        server = json.loads(stritem)
                        event = HeartbeatEvent(event_type=HeartbeatEventType.NEW,
                                               namespace=namespace,
                                               service_name=server['service_name'],
                                               host_ip=server['host_ip'],
                                               host_name=server['host_name'],
                                               timestamp=newdict[stritem])
                        self.callback(event)

                    for stritem in oldset - newset:
                        server = json.loads(stritem)
                        event = HeartbeatEvent(event_type=HeartbeatEventType.LOST,
                                               namespace=namespace,
                                               service_name=server['service_name'],
                                               host_ip=server['host_ip'],
                                               host_name=server['host_name'],
                                               timestamp=olddict[stritem])
                        self.callback(event)

                    self.service_mapping[namespace] = newdict

    def start(self):
        self.running = True

    def stop(self):
        self.running = False



import json
import time
from typing import List

import redis

from rosetta_dispatcher.idtool import idtool
from rosetta_dispatcher.model.dispatch_request_model import DispatchRequestModel
from rosetta_dispatcher.model.dispatch_response_model import DispatchResponseModel


class DispatchServer:
    def __init__(self, redis_host: str, redis_port: int, response_timeout: int = 60):
        self.redis_host = redis_host
        self.redis_port = redis_port
        self.response_timeout = response_timeout
        self.pool = redis.ConnectionPool(host=redis_host, port=redis_port, decode_responses=True)

    def preprocess_request(self, strrequest: str) -> DispatchRequestModel:
        if strrequest:
            dict_request = json.loads(strrequest)
            request = DispatchRequestModel.parse_obj(dict_request)
            if request.correlation_id:
                start_time = idtool.get_timestamp(request.correlation_id)
                ts_now = time.time()
                if ts_now < start_time + request.timeout:
                    return request

        return None

    def __fetch_one__(self, r: redis.Redis, service_queue: str):
        return

    def __batch_fetch__(self, r: redis.Redis, service_queue: str, batch_count: int = 16):
        p = r.pipeline()
        p.multi()  # starts transactional block of pipeline
        items = p.lrange(service_queue, 0, batch_count - 1)
        p.ltrim(service_queue, batch_count, -1)
        messages, trim_success = p.execute()  # ends transactional block of pipeline

        result = []
        for strrequest in messages:
            request = self.preprocess_request(strrequest)
            if request:
                result.append(request)
            else:
                print('drop time out request')
        return result

    def fetch(self, service_queue: str, batch_count=16, timeout: int = 10) -> List[DispatchRequestModel]:
        r = redis.Redis(connection_pool=self.pool)
        batch_requests = self.__batch_fetch__(r, service_queue, batch_count)
        if batch_requests:
            return batch_requests

        result = []
        # block and wait data.
        trequest = r.blpop(keys=service_queue, timeout=timeout)
        # None data fetched.
        if not trequest:
            return result

        request = self.preprocess_request(trequest[1])
        if request:
            result.append(request)

        batch_requests = self.__batch_fetch__(r, service_queue, batch_count-1)
        if batch_requests:
            result += batch_requests

        return result

    def send_response(self, response_queue: str, response: DispatchResponseModel):
        r = redis.Redis(connection_pool=self.pool)
        data = response.dict(exclude_none=True)
        r.rpush(response_queue, json.dumps(data, ensure_ascii=True))
        r.expire(response_queue, time=self.response_timeout)

    def batch_send_response(self, response_queue_list: List[DispatchRequestModel], response_list: List[DispatchResponseModel]):
        if len(response_queue_list) != len(response_queue_list):
            raise Exception("response queue list is not equal response_list")

        r = redis.Redis(connection_pool=self.pool)
        p = r.pipeline()
        for request, response in zip(response_queue_list, response_list):
            data = response.dict(exclude_none=True)
            p.rpush(request.reply_to, json.dumps(data, ensure_ascii=True))
            p.expire(request.reply_to, time=self.response_timeout)
        p.execute()  # ends transactional block of pipeline



import time

from heartbeat.heatbeat_reporter import HeartbeatReporter

host = '61.160.36.168'
port = 4032

hbr = HeartbeatReporter(namespace="SERVICE_NMT_ZH_EN",
                        service_name="heartbeat_service_name",
                        host_ip="0.0.0.0",
                        host_name="heatbeat_host_name",
                        redis_host=host,
                        redis_port=port,
                        interval=1)
hbr.start()

while True:
    time.sleep(1)
